#include <reg51.h>
#include "stdutils.h"





/********************************************
              NEC IR Remote Codes 
*********************************************/  
#define IR_CODE_POWER      0x01FE48B7
#define IR_CODE_MODE       0x01FE58A7
#define IR_CODE_MUTE       0x01FE7887
#define IR_CODE_BACKW      0x01FE40BF 
#define IR_CODE_FWD        0x01FEC03F
#define IR_CODE_PLAY_PAUSE 0x01FE807F
#define IR_CODE_VOL_MINUS  0x01FEA05F
#define IR_CODE_VOL_PLUS   0x01FE609F
#define IR_CODE_EQ         0x01FE20DF
#define IR_CODE_RPT        0x01FE10EF
#define IR_CODE_U/SD       0x01FE906F
#define IR_CODE_0          0x01FEE01F
#define IR_CODE_1          0x01FE50AF
#define IR_CODE_2          0x01FED827
#define IR_CODE_3          0x01FEF807
#define IR_CODE_4          0x01FE30CF
#define IR_CODE_5          0x01FEB04F
#define IR_CODE_6          0x01FE708F
#define IR_CODE_7          0x01FE00FF
#define IR_CODE_8          0x01FEF00F
#define IR_CODE_9          0x01FE9867
void Remote_Init(void);
uint32_t Getkey(void);



uint32_t Pattern=0,decKey=0;
uint8_t timerLength;
uint8_t Count=0;
char Pulse=0;

void timer0_isr() interrupt 1
{
    if(Count<50)
        Count++;

    TH0 = 0xFC;   // Reload the timer value for 1ms Delay
    TL0 = 0x67;
}



void externalIntr0_ISR()  interrupt 0
{
    timerLength = Count;
    Count = 0;

    TH0 = 0xFC;  // Reload the timer value for 1ms Delay
    TL0 = 0x67;

    Pulse++;

    if((timerLength>=50)) // If the pulse width is greater than 50ms,then start of the frame
    { 
        Pulse = -2; // First 2 counts will be skipped,so pulse count is set to minus 2
        Pattern = 0;
    }
    else if((Pulse>=0) && (Pulse<32)) //Accumulate the bit values between 0-31.
    {  
        if(timerLength>=2)                      //pulse width greater than 2ms will be taken as as LOGIC1
        {
            Pattern |=(uint32_t)1<<(31-Pulse);
        }
else
        {

        }
        
    }
    else if(Pulse>=32)                  // End of frame as 32 pulses are received
    {
        decKey = Pattern;               
        Pulse = 0;
    }
}



void Remote_Init(void)
{
    TMOD |= 0x01;  // Timer0 MODE1(16-bit timer)
    TH0 = 0xFC;    // Timer value for 1ms at 11.0592Mhz clock
    TL0 = 0x67;
    TR0 = 1;       // Start the Timer
    ET0 = 1;       // Enable the Timer0 Interrupt
    
    IT0 = 1;       // Configure INT0 falling edge interrupt
    EX0 = 1;       // Enable the INT0 External Interrupt
    
    EA  = 1;       // Enable the Global Interrupt bit	  
}


uint32_t Getkey(void)
{
	 uint32_t keyCode;
	 keyCode = decKey;
	 decKey = 0x00;

	 return keyCode;	   
} 

sbit RELAY_PIN1 = P0^0;
sbit RELAY_PIN2 = P0^1;

void main()
{
    uint32_t key;
	RELAY_PIN1 = 1;	  // Turn OFF the relay during Init
	RELAY_PIN2 = 1;	 
    Remote_Init();  
    while(1)
    {
        key = Getkey();  // Read the key press, 
        switch(key)
        {
        case IR_CODE_1: RELAY_PIN1 = 1; break; /* Turn OFF Relay if 1 is pressed*/
        case IR_CODE_0: RELAY_PIN1= 0; break; /* Turn ON Relay if 0 is pressed*/
        case IR_CODE_2: RELAY_PIN2 = 0; break; /* Turn On if 1 is pressed*/
        case IR_CODE_3: RELAY_PIN2= 1; break; /* Turn OFF Relay if 0 is pressed*/
        default: break;
        }
    }		
}	  